#!/bin/bash
#
# Adapt then export the sysroots into tarball
# Generate a script to install the tarball and set up environment for SDK
#

if [ -r "$PLATFORM_LOC/scripts/utils.sh" ]; then
	source "$PLATFORM_LOC/scripts/utils.sh"
else
	source "$script_lib_dir/utils.sh"
fi

MACHINE=${machine}
outputDir=${PLATFORM_LOC}/sdk

# Create temporary workdir
WORKDIR=$(mktemp -d -t tmp-XXXXXXXXXX)
trap "rm -rf ${WORKDIR}" EXIT
cd ${WORKDIR}

SDK_NAME="SW4Linux-SDK-${MACHINE}"
SDK_ENV_SCRIPT="setup-env-${SDK_NAME}.sh"
SDK_INS_SCRIPT="install-${SDK_NAME}.sh"
SDK_DIR=${WORKDIR}/${SDK_NAME}
SDK_SYSROOTS_DIR=${SDK_DIR}/sysroots
mkdir -p ${SDK_SYSROOTS_DIR}

# Export Native sysroot
if [ ! -d "${NATIVE_SYSROOT_LOC}" ]; then
	fatal 1 "Native sysroot does not exist"
fi

echo -n "Exporting native sysroots into SDK..."
cp -r ${NATIVE_SYSROOT_LOC} ${SDK_SYSROOTS_DIR}
SDK_NATIVE_SYSROOT_DIRNAME="$(basename ${NATIVE_SYSROOT_LOC})"
SDK_CROSS_COMPILER_PATH="${CROSS_COMPILER_PATH##${NATIVE_SYSROOT_LOC}}"
SDK_CROSS_COMPILER_LOC="${CROSS_COMPILER_LOC##${NATIVE_SYSROOT_LOC}}"
SDK_CROSS_PREFIX="${CROSS_PREFIX}"
echo "... done"

# Export Target sysroot
if [ ! -d "${TARGET_SYSROOT_LOC}" ]; then
	fatal 1 "Target sysroot does not exist"
fi

echo -n "Exporting target sysroots into SDK..."
cp -r ${TARGET_SYSROOT_LOC} ${SDK_SYSROOTS_DIR}
SDK_TARGET_SYSROOT_DIRNAME="$(basename $TARGET_SYSROOT_LOC)"
echo "... done"


# Generate environmnent script that define every required variables to use the SDK
echo -n "Creating environment setup script..."
cat <<- EOF >> ${SDK_DIR}/${SDK_ENV_SCRIPT}
#!/bin/bash

# Check whether user had supplied -h or --help . If yes display usage
if [[ ( \$@ == "--help") ||  \$@ == "-h" ]]
then 
	echo "Usage: source \$0"
	exit 0
fi

# Export environment variable
echo "Sourcing environment variables..."
export SDK_LOC=\$( cd -- "\$( dirname -- "\${BASH_SOURCE[0]}" )" &> /dev/null && pwd )
export SYSROOT_LOC="\${SDK_LOC}/sysroots/${SDK_TARGET_SYSROOT_DIRNAME}"
export NATIVE_SYSROOT_LOC="\${SDK_LOC}/sysroots/${SDK_NATIVE_SYSROOT_DIRNAME}"
export TARGET_SYSROOT_LOC="\${SDK_LOC}/sysroots/${SDK_TARGET_SYSROOT_DIRNAME}"
export arch_options="${arch_options}"
export arch_as_options="\${arch_options} ${as_options}"
export arch_gcc_options="\${arch_options} ${gcc_options}"
export arch_gPP_options="\${arch_options} ${gPP_options}"
export arch_ld_options="\${arch_options} ${ld_options}"
export includedir="\${SYSROOT_LOC}/usr/include"
export arch_name="${arch_name}"
export arch_triplet="${arch_triplet}"
export arch_compiler_prefix="${arch_name}-${arch_triplet}"
export CROSS_PREFIX=${SDK_CROSS_PREFIX}
export CFLAGS="-O2 -pipe"
export CCLD=${arch_compiler_prefix}-gcc
export STRIP=${arch_compiler_prefix}-strip
export CXXLD=${arch_compiler_prefix}-g++
export RANLIB=${arch_compiler_prefix}-ranlib
export OBJDUMP=${arch_compiler_prefix}-objdump
export AS=${arch_compiler_prefix}-as
export AR=${arch_compiler_prefix}-ar
export NM=${arch_compiler_prefix}-nm
export CROSS_COMPILE=${arch_compiler_prefix}-
export STRINGS=${arch_compiler_prefix}-strings
export compiler_prefix=${SDK_CROSS_PREFIX}
export CROSS_COMPILER_PATH="\${NATIVE_SYSROOT_LOC}${SDK_CROSS_COMPILER_PATH}"
export compiler_loc="\${NATIVE_SYSROOT_LOC}${SDK_CROSS_COMPILER_LOC}"
export CPP="${arch_compiler_prefix}-gcc -E"
export LD=${arch_compiler_prefix}-ld
export CC=${arch_compiler_prefix}-gcc
export READELF=${arch_compiler_prefix}-readelf         

# Export new PATH environment variable
export PATH=\${NATIVE_SYSROOT_LOC}/bin:\${NATIVE_SYSROOT_LOC}/usr/bin:\${CROSS_COMPILER_PATH}:\${PATH}

echo "Rewriting old hard path (it might takes some times)..."
# Find and replace all the remaining hard path
find \${SDK_LOC}/sysroots -type f ! -perm /0111 -print0 | xargs -0 sed -i -e "s|@@SDK_PATH_PLACEHOLDER@@|\${SDK_LOC//\//\\/}|g"

EOF

chmod +x ${SDK_DIR}/${SDK_ENV_SCRIPT}
echo "... done"

# Replace absolute path to platform into place holder
# Those place holders will be replaced by local when sourcing SDK script
echo -n "Package SDK..."
find ${SDK_DIR} -type f ! -perm /0111 -print0 | xargs -0 sed -i -e "s|${PLATFORM_LOC//\//\\/}|@@SDK_PATH_PLACEHOLDER@@|g"
tar czf ${SDK_NAME}.tar.gz ${SDK_NAME}
echo "... done"

# Generate installation script
cat <<- EOF >> ${SDK_INS_SCRIPT}
#!/bin/bash
#
# SW4Linux SDK install script
# 
command=\$0
command_line="\$0 \$*"
PAYLOAD_LINE=\`awk '/^__PAYLOAD__/ {print NR + 1; exit 0; }' \$0\`
destDir=

usage() { 
	echo "SW4Linux SDK installer"
	echo "This script must be run with super-user privileges if you want to install into a root directory"
	echo "Usage: \$command [OPTION] "
	echo "Options:"
	echo "	-d --dest=DIR			DIR \tSet the installation directory DIR"
	echo "	-h --help			Display this help"
} 

error() { 
	echo "SW4Linux SDK installer"
	echo "Error: invalid options"
	echo "Try '\$command --help' for more information"
} 

extract_sdk() {
	local destDir=\$1
	if [ ! -d \$destDir ]; then
		mkdir -p \$destDir
	fi
	
	echo "Extracting SDK into \$destDir....."
	tail -n+\$PAYLOAD_LINE \$0 | tar xzv -C \$destDir
	echo "To use the SDK, please source the setup environment script: "
	echo " $> . \$destDir/${SDK_NAME}/${SDK_ENV_SCRIPT}"
}

while [ \$# -ne 0 ]; do
	opt="\$1"
	case \$opt in
		-h | --help) 
			usage 
			exit 0
			;; 
		-d) 
			if [ \$# -gt 1 ]; then
				shift
				destDir="\$1"
			else
				echo "Option '\$opt' need a value"
				exit 1
			fi
			;;
		--dest=*)
			destDir=\${opt#*=}
			;;
		*) 
			error 
			exit 1
			;; 
    esac
done

echo "SW4Linux SDK installer:"
echo "======================="

if [ -n \$destDir ]; then
	read -e -r -p "Enter target directory for SDK (default: /opt/sw4linux/sdk): " destPath
	if [ -z \$destPath ]; then
		destDir="/opt/sw4linux/sdk"
	else
		destDir="\$destPath"	
	fi
fi

read -e -r -p "You are about to install the SDK to \${destDir}. Continue [Y/n]? " yn
case \$yn in 
	[yY])
		extract_sdk \$destDir
		;;
	*)
		exit 0
		;;
esac
exit 0

__PAYLOAD__
EOF

# Create output directory
mkdir -p ${outputDir}

# Append sdk to install script
cat ${SDK_NAME}.tar.gz >> ${SDK_INS_SCRIPT}

# Move install script to output directory
if [ -f "${outputDir}/${SDK_INS_SCRIPT}" ]; then
	rm ${outputDir}/${SDK_INS_SCRIPT}
fi
mv ${SDK_INS_SCRIPT} ${outputDir}
chmod a+x ${outputDir}/${SDK_INS_SCRIPT}

# Copy SDK script from working directory to destination
log "Please find SDK in ${outputDir}"